import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _extends from "@babel/runtime/helpers/extends";
import _typeof from "@babel/runtime/helpers/typeof";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";

/* eslint-disable react-hooks/exhaustive-deps */
import React, { useEffect, useReducer } from 'react';
import PropTypes from 'prop-types';
import clsx from 'clsx';
import isEqual from 'lodash/isEqual';
import fnToString from '../utils/fn-to-string';
import reducer from './reducer';
import useIsMounted from '../hooks/use-is-mounted';

var getSelectValue = function getSelectValue(stateValue, simpleValue, isMulti, allOptions) {
  return simpleValue ? allOptions.filter(function (_ref) {
    var value = _ref.value;
    return isMulti ? stateValue.includes(value) : isEqual(value, stateValue);
  }) : stateValue;
};

var handleSelectChange = function handleSelectChange(option, simpleValue, isMulti, onChange) {
  var sanitizedOption = !option && isMulti ? [] : option;
  return simpleValue ? onChange(isMulti ? sanitizedOption.map(function (item) {
    return item.value;
  }) : sanitizedOption ? sanitizedOption.value : undefined) : onChange(sanitizedOption);
};

var Select = function Select(_ref2) {
  var invalid = _ref2.invalid,
      classNamePrefix = _ref2.classNamePrefix,
      simpleValue = _ref2.simpleValue,
      isMulti = _ref2.isMulti,
      pluckSingleValue = _ref2.pluckSingleValue,
      propsOptions = _ref2.options,
      loadOptions = _ref2.loadOptions,
      loadingMessage = _ref2.loadingMessage,
      loadingProps = _ref2.loadingProps,
      selectVariant = _ref2.selectVariant,
      updatingMessage = _ref2.updatingMessage,
      noOptionsMessage = _ref2.noOptionsMessage,
      value = _ref2.value,
      _onChange = _ref2.onChange,
      loadOptionsChangeCounter = _ref2.loadOptionsChangeCounter,
      SelectComponent = _ref2.SelectComponent,
      noValueUpdates = _ref2.noValueUpdates,
      props = _objectWithoutProperties(_ref2, ["invalid", "classNamePrefix", "simpleValue", "isMulti", "pluckSingleValue", "options", "loadOptions", "loadingMessage", "loadingProps", "selectVariant", "updatingMessage", "noOptionsMessage", "value", "onChange", "loadOptionsChangeCounter", "SelectComponent", "noValueUpdates"]);

  var _useReducer = useReducer(reducer, {
    isLoading: false,
    options: propsOptions,
    promises: {},
    isInitialLoaded: false
  }),
      _useReducer2 = _slicedToArray(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  var isMounted = useIsMounted();

  var updateOptions = function updateOptions() {
    dispatch({
      type: 'startLoading'
    });
    return loadOptions().then(function (data) {
      if (isMounted.current) {
        if (!noValueUpdates) {
          if (value && Array.isArray(value)) {
            var _selectValue = value.filter(function (value) {
              return _typeof(value) === 'object' ? data.find(function (option) {
                return value.value === option.value;
              }) : data.find(function (option) {
                return value === option.value;
              });
            });

            _onChange(_selectValue.length === 0 ? undefined : _selectValue);
          } else if (value && !data.find(function (_ref3) {
            var internalValue = _ref3.value;
            return internalValue === value;
          })) {
            _onChange(undefined);
          }
        }

        dispatch({
          type: 'updateOptions',
          payload: data
        });
      }
    });
  };

  useEffect(function () {
    if (loadOptions) {
      updateOptions();
    }

    dispatch({
      type: 'initialLoaded'
    });
  }, []);
  var loadOptionsStr = loadOptions ? fnToString(loadOptions) : '';
  useEffect(function () {
    if (loadOptionsStr && state.isInitialLoaded) {
      updateOptions();
    }
  }, [loadOptionsStr, loadOptionsChangeCounter]);
  useEffect(function () {
    if (state.isInitialLoaded) {
      if (!noValueUpdates && value && !propsOptions.map(function (_ref4) {
        var value = _ref4.value;
        return value;
      }).includes(value)) {
        _onChange(undefined);
      }

      dispatch({
        type: 'setOptions',
        payload: propsOptions
      });
    }
  }, [propsOptions]);

  var renderNoOptionsMessage = function renderNoOptionsMessage() {
    return Object.values(state.promises).some(function (value) {
      return value;
    }) ? function () {
      return updatingMessage;
    } : function () {
      return noOptionsMessage;
    };
  };

  if (state.isLoading) {
    return /*#__PURE__*/React.createElement(SelectComponent, _extends({}, props, {
      classNamePrefix: classNamePrefix,
      isDisabled: true,
      isFetching: true,
      placeholder: loadingMessage,
      options: state.options,
      onChange: function onChange() {}
    }, loadingProps, {
      noOptionsMessage: renderNoOptionsMessage()
    }));
  }

  var onInputChange = function onInputChange(inputValue) {
    if (inputValue && loadOptions && state.promises[inputValue] === undefined && props.isSearchable) {
      dispatch({
        type: 'setPromises',
        payload: _defineProperty({}, inputValue, true)
      });
      loadOptions(inputValue).then(function (options) {
        if (isMounted.current) {
          dispatch({
            type: 'setPromises',
            payload: _defineProperty({}, inputValue, false),
            options: options
          });
        }
      })["catch"](function (error) {
        dispatch({
          type: 'setPromises',
          payload: _defineProperty({}, inputValue, false)
        });
        throw error;
      });
    }
  };

  var selectValue = pluckSingleValue ? isMulti ? value : Array.isArray(value) && value[0] ? value[0] : value : value;
  return /*#__PURE__*/React.createElement(SelectComponent, _extends({
    className: clsx(classNamePrefix, {
      'has-error': invalid
    })
  }, props, {
    isDisabled: props.isDisabled || props.isReadOnly,
    options: state.options,
    classNamePrefix: classNamePrefix,
    isMulti: isMulti,
    value: getSelectValue(selectValue, simpleValue, isMulti, state.options),
    onChange: function onChange(option) {
      return handleSelectChange(option, simpleValue, isMulti, _onChange);
    },
    onInputChange: onInputChange,
    isFetching: Object.values(state.promises).some(function (value) {
      return value;
    }),
    noOptionsMessage: renderNoOptionsMessage(),
    hideSelectedOptions: false,
    closeMenuOnSelect: !isMulti
  }));
};

Select.propTypes = {
  options: PropTypes.array,
  onChange: PropTypes.func,
  classNamePrefix: PropTypes.string,
  invalid: PropTypes.bool,
  simpleValue: PropTypes.bool,
  isMulti: PropTypes.bool,
  pluckSingleValue: PropTypes.bool,
  value: PropTypes.any,
  placeholder: PropTypes.string,
  loadOptionsChangeCounter: PropTypes.number,
  isDisabled: PropTypes.bool,
  isReadOnly: PropTypes.bool,
  loadOptions: PropTypes.func,
  loadingMessage: PropTypes.node,
  loadingProps: PropTypes.object,
  selectVariant: PropTypes.string,
  updatingMessage: PropTypes.node,
  noOptionsMessage: PropTypes.node,
  isSearchable: PropTypes.bool,
  SelectComponent: PropTypes.elementType.isRequired,
  noValueUpdates: PropTypes.bool
};
Select.defaultProps = {
  options: [],
  invalid: false,
  simpleValue: true,
  pluckSingleValue: true,
  placeholder: 'Choose...',
  isSearchable: false,
  isClearable: false
};
export default Select;